# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
# See LICENSE.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
CVPySDK (Developer SDK - Python) is a Python Package for Commvault Software.

CVPySDK uses Commvault REST API to perform operations on a Commcell.

CVPySDK is available on GitHub (https://github.com/CommvaultEngg/cvpysdk).

CVPySDK is compatible with both Python 2 and Python 3

CVPySDK requires the following Python packages to run:
    "requests"

    "future"

    "xmltodict"

And Commvault Software v11 SP7 or later release with WebConsole installed

"""

__author__ = 'Commvault Systems Inc.'
__version__ = '11.11'
